'use strict';

process.env['PATH'] = process.env['PATH'] + ':' + process.env['LAMBDA_TASK_ROOT'];

const execFile = require('child_process').execFile;
const request = require('request-promise');
const Promise = require('bluebird');

const phantomUrl = 'http://localhost:9998/';
let phantom;

function startPhantomServer(proxy, proxyType){
  if (proxyType != 'socks5') proxyType = 'http';
  try {
    phantom = execFile('./phantomjs', [`--proxy=${proxy}`, `--proxy-type=${proxyType}`, 'phantom-server.js']);

    phantom.stdout.on('data', (data) => {
      console.log(data + '');
    });

    phantom.stderr.on('data', (data) => {
      console.log('Phantom error  ---:> ' + data);
    });

    phantom.on('close', (code) => {
      console.log('Child process exited with code ' + code);
    });


  } catch(err) {
    console.log('err', err);
    process.exit(1);
  }
}

function isPhantomServerReady(){
  console.log('Checking the server ready status...');
  return new Promise(function(resolve, reject) {
    setTimeout(function () {
      request(phantomUrl)
      .then(() => resolve(console.log('Server is ready')))
      .catch((err) => {
        return resolve(isPhantomServerReady());
      })
    }, 100);
  });
}

exports.handler = function(event, context, callback) {
  console.log('Received event from handler:', JSON.stringify(event, null, 2));
  if (!event.proxy){
    return  callback('Missing proxy param');
  }
  startPhantomServer(event.proxy, event.proxyType);
  isPhantomServerReady()
  .then(function(){
    request.post({
      headers: {'content-type' : 'application/x-www-form-urlencoded'},
      url:     phantomUrl,
      form: {
        link: event.link,
        eval: event.eval,
        options: event.options
      }
    })
    .then(function(response) {
      phantom.kill();

      callback(null,  response);
    })
    .catch(function(error) {
      phantom.kill();
      //if (typeof(error) === 'string') error = JSON.parse(error);
      //if (error.errorMessage) error.errorMessage = JSON.parse(error.errorMessage);
      //console.log(error);

      callback(error);
    });
  });
};