'use strict';


/**
 *
 */
class ScrapeStatus {
  /**
   *
   */
  constructor(config) {
    if (!config.urls) throw new Error('Scrape status provided no urls.');

    Object.assign(this, {
      urls: config.urls,
      indexed: {},
      successful: [],
      failed: [],
      remaining: config.urls.length
    });

    this.onUpdateCB = config.onUpdate;

    config.urls.forEach((url) => {
      const urlConfig = {url, maxAttempts: config.maxAttempts};
      this.indexed[url] = new UrlStatus(urlConfig);
    });
  }



  /** 
   *
   */
  update(url, proxyIP, result) {
    this.indexed[url].markAttempt();
    result && this.indexed[url].setResult(result);

    if (this.onUpdate) {
      const event = Object.assign({
        success: !!result,
        proxyIP
      }, this.indexed[url]);

      if (result) {
        event.result = result;
        this.successful.push(url);
        this.successful.sort();
        this.remaining--;
      }

      this.onUpdate(event, this);
    };
  }

  /**
   *
   */
  onUpdate(event) {
    if (this.onUpdateCB) this.onUpdateCB(event, this);
  }


  /**
   *
   */
  hasExceededAttempts(url) {
    return this.indexed[url].hasExceededAttempts();
  }
}



/**
 *
 */
const MAX_ATTEMPTS = 50;

/**
 *
 */
class UrlStatus {
  /**
   *
   */
  constructor(config) {
    Object.assign(this, {
      url: config.url,
      attempts: 0,
      maxAttempts: config.maxAttempts || MAX_ATTEMPTS,
      result: null
    });
  }

  /**
   *
   */
  markAttempt() {
    this.attempts++;
  }

  /**
   *
   */
  setResult(result) {
    this.result = result;
  }

  /**
   *
   */
  hasExceededAttempts() {
    return this.attempts >= this.maxAttempts;
  }
}

module.exports = ScrapeStatus;
